
#pragma once

#include "AlivcDef.h"
#include "sdk_headers/AliMediaDownloader.h"
#include "ui_AlivcDownloadItemWidget.h"
#include <QWidget>
#include <memory>
#include "QListWidget"

using namespace alivc_player;

class AlivcDownloadItemWidget : public QWidget, public IAVDListener {
    Q_OBJECT
public:
    AlivcDownloadItemWidget(QWidget *parent);
    ~AlivcDownloadItemWidget();
    void SetSourceInfo(const DownloadSourceInfo &sourceInfo, const QString saveDir);
    DownloadSourceInfo GetSourceInfo();
    void SetListItem(QListWidgetItem *listItem);
    QListWidgetItem *GetListItem();
    void Start();
    void Delete();

    //IAVDListener
    void onPrepared(AliMediaDownloader *downloader, AVPMediaInfo *mediaInfo) override;
    void onError(AliMediaDownloader *downloader, AVPErrorModel *errorModel) override;
    void onDownloadingProgress(AliMediaDownloader *downloader, int percent) override;
    void onProcessingProgress(AliMediaDownloader *downloader, int percent) override;
    void onCompletion(AliMediaDownloader *downloader) override;

signals:
    void SgnPrepared(AVPMediaInfo* mediaInfo);
    void SgnError(QString error);
    void SgnDownloadingProgress(int percent);
    void SgnProcessingProgress(int percent);
    void SgnCompletion();

private slots:
    void on_btnStart_clicked();
    void on_btnStop_clicked();
    void on_btnPlay_clicked();
    void on_btnDelete_clicked();
    void OnPreparedUI(AVPMediaInfo* mediaInfo);
    void OnErrorUI(QString error);
    void OnDownloadingProgressUI(int percent);
    void OnProcessingProgressUI(int percent);
    void OnCompletionUI();

private:
    void InitEvent();
    void Prepare();
    std::unique_ptr<Ui::AlivcDownloadItemWidget> ui;
    DownloadSourceInfo mSourceInfo;
    AliMediaDownloader *mMediaDownloader = nullptr;
    QString mSaveDir;
    bool mDownloaded = false;
    QListWidgetItem *mListItem = nullptr;
};
