#include "AlivcDownloadPrepareDlg.h"
#include "QJsonDocument"
#include "QMessageBox"

const static QString GetPlayAuthInfoUrl = "https://alivc-demo.aliyuncs.com/player/getVideoPlayAuth";

AlivcDownloadPrepareDlg::AlivcDownloadPrepareDlg(QWidget *parent) : QDialog(parent), ui(new Ui::AlivcDownloadPrepareDlg)
{
    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
    ui->setupUi(this);
    InitEvent();
    EnableStsControls(true);
    EnablePlayAuthControls(false);
}

AlivcDownloadPrepareDlg::~AlivcDownloadPrepareDlg()
{
    if (mAppServerApi) {
        delete mAppServerApi;
        mAppServerApi = nullptr;
    }
    if (mMediaDownloader) {
        mMediaDownloader->stop();
        delete mMediaDownloader;
        mMediaDownloader = nullptr;
    }
}

DownloadSourceInfo AlivcDownloadPrepareDlg::GetDownloadSourceInfo()
{
    DownloadSourceInfo ret;
    ret.vid = ui->editVid->text();
    ret.accessKeyId = ui->editAccessKeyId->text();
    ret.accessKeySecret = ui->editAccessKeySecret->text();
    ret.playAuth = ui->editPlayAuth->text();
    ret.region = ui->editRegion->text();
    ret.securityToken = ui->editSecurityToken->text();
    ret.sourceType = ui->btnStsDownload->isChecked() ? DownloadSourceTypeSts : DownloadSourceTypePlayAuth;
    ret.trackDesc = ui->comboTrack->currentText();
    ret.trackIndex = ui->comboTrack->currentData().toInt();
    return ret;
}

void AlivcDownloadPrepareDlg::CheckAuthInfo()
{
    if (!ui->editPlayAuth->text().isEmpty()) {
        ui->editPlayAuth->setText("");
    }
    if (ui->btnPlayAuthDownload->isChecked()) {
        ui->comboTrack->clear();
    }
    QString expiration = ui->editExpiration->text();
    if (!expiration.isEmpty()) {
        QDateTime expirationDate = QDateTime::fromString(expiration, Qt::ISODate);
        expirationDate = expirationDate.addSecs(-3600);
        QDateTime currentDate = QDateTime::currentDateTime();
        if (expirationDate < currentDate) {
            ui->editAccessKeyId->setText("");
            ui->editAccessKeySecret->setText("");
            ui->editSecurityToken->setText("");
            ui->editExpiration->setText("");
            ui->comboTrack->clear();
        }
    }
}

void AlivcDownloadPrepareDlg::onPrepared(AliMediaDownloader *downloader, AVPMediaInfo *mediaInfo)
{
    mMediaInfo = *mediaInfo;
    emit SgnPrepared();
}

void AlivcDownloadPrepareDlg::onError(AliMediaDownloader *downloader, AVPErrorModel *errorModel)
{
    QString strError = QString::fromStdString(errorModel->message);
    emit SgnError(strError);
}

void AlivcDownloadPrepareDlg::onDownloadingProgress(AliMediaDownloader *downloader, int percent)
{}

void AlivcDownloadPrepareDlg::onProcessingProgress(AliMediaDownloader *downloader, int percent)
{}

void AlivcDownloadPrepareDlg::onCompletion(AliMediaDownloader *downloader)
{}

void AlivcDownloadPrepareDlg::on_btnDone_clicked()
{
    if (ui->comboTrack->count() == 0) {
        QMessageBox::information(NULL, "Error", QStringLiteral("׼"), QMessageBox::Ok, QMessageBox::NoButton);
        return;
    }
    accept();
}

void AlivcDownloadPrepareDlg::on_btnStsDownload_clicked()
{
    ui->btnPlayAuthDownload->setChecked(false);
    EnableStsControls(true);
    EnablePlayAuthControls(false);
}

void AlivcDownloadPrepareDlg::on_btnPlayAuthDownload_clicked()
{
    ui->btnStsDownload->setChecked(false);
    EnableStsControls(false);
    EnablePlayAuthControls(true);
    QString url = GetPlayAuthInfoUrl;
    QString vid = ui->editVid->text();
    if (!vid.isEmpty()) {
        url.append("?videoId=");
        url.append(vid);
    }
    ui->editPlayAuthUrl->setText(url);
}

void AlivcDownloadPrepareDlg::on_btnGetStsInfo_clicked()
{
    if (mAppServerApi == nullptr) {
        mAppServerApi = new AlivcAppServerBaseAPI(this);
    }
    mAppServerApi->Get(ui->editStsUrl->text(), [&](int statusCode, QByteArray data) {
        QJsonParseError jsonError;
        QJsonDocument jsonDocument = QJsonDocument::fromJson(data, &jsonError);
        if (jsonError.error != QJsonParseError::NoError || !jsonDocument.isObject()) {
            return;
        }

        QVariantMap result = jsonDocument.toVariant().toMap();
        QVariant dataValue = result["data"];
        QVariantMap dataMap = dataValue.toMap();
        ui->editAccessKeyId->setText(dataMap["accessKeyId"].toString());
        ui->editAccessKeySecret->setText(dataMap["accessKeySecret"].toString());
        ui->editSecurityToken->setText(dataMap["securityToken"].toString());
        ui->editExpiration->setText(dataMap["expiration"].toString());
    });
}

void AlivcDownloadPrepareDlg::on_btnGetPlayAuthInfo_clicked()
{
    if (mAppServerApi == nullptr) {
        mAppServerApi = new AlivcAppServerBaseAPI(this);
    }
    mAppServerApi->Get(ui->editPlayAuthUrl->text(), [&](int statusCode, QByteArray data) {
        QJsonParseError jsonError;
        QJsonDocument jsonDocument = QJsonDocument::fromJson(data, &jsonError);
        if (jsonError.error != QJsonParseError::NoError || !jsonDocument.isObject()) {
            return;
        }

        QVariantMap result = jsonDocument.toVariant().toMap();
        QVariant dataValue = result["data"];
        QVariantMap dataMap = dataValue.toMap();
        ui->editPlayAuth->setText(dataMap["playAuth"].toString());
        QVariantMap metaMap = dataMap["videoMeta"].toMap();
        ui->editVid->setText(metaMap["videoId"].toString());
    });
}

void AlivcDownloadPrepareDlg::on_btnPrepare_clicked()
{
    QString vid = ui->editVid->text();
    if (vid.isEmpty()) {
        QMessageBox::information(NULL, "Error", QStringLiteral("vid"), QMessageBox::Ok, QMessageBox::NoButton);
        return;
    }
    QString region = ui->editRegion->text();

    if (mMediaDownloader == nullptr) {
        mMediaDownloader = AliMediaDownloader::CreateMediaDownloader();
        mMediaDownloader->setListener(this);
    }

    if (ui->btnStsDownload->isChecked()) {
        QString accessKeyId = ui->editAccessKeyId->text();
        QString accessKeySecret = ui->editAccessKeySecret->text();
        QString securityToken = ui->editSecurityToken->text();
        if (accessKeyId.isEmpty() || accessKeySecret.isEmpty() || securityToken.isEmpty()) {
            QMessageBox::information(NULL, "Error", QStringLiteral("stsϢ"), QMessageBox::Ok, QMessageBox::NoButton);
            return;
        }
        AVPVidStsSource vidSource;
        vidSource.initWithVid(vid.toLocal8Bit(), accessKeyId.toLocal8Bit(), accessKeySecret.toLocal8Bit(), securityToken.toLocal8Bit(),
                              region.toLocal8Bit(), nullptr);
        mMediaDownloader->prepareWithVid(&vidSource);
    } else if (ui->btnPlayAuthDownload->isChecked()) {
        QString playAuth = ui->editPlayAuth->text();
        if (playAuth.isEmpty()) {
            QMessageBox::information(NULL, "Error", QStringLiteral("playAuthϢ"), QMessageBox::Ok, QMessageBox::NoButton);
            return;
        }
        AVPVidAuthSource authSource;
        authSource.initWithVid(vid.toLocal8Bit(), playAuth.toLocal8Bit(), region.toLocal8Bit(), nullptr);
        mMediaDownloader->prepareWithPlayAuth(&authSource);
    } else {
        assert(false);
    }
    
}

void AlivcDownloadPrepareDlg::on_editVid_textChanged()
{
    ui->comboTrack->clear();
}

void AlivcDownloadPrepareDlg::OnPreparedUI()
{
    ui->comboTrack->blockSignals(true);
    ui->comboTrack->clear();
    for (int i = 0; i < mMediaInfo.trackCount; i++) {
        AVPTrackInfo *track = mMediaInfo.tracks[i];
        if (track && track->trackType == AVPTRACK_TYPE_SAAS_VOD) {
            int index = ui->comboTrack->count();
            QString sizeString;
            int fileSize = track->vodFileSize;
            if (fileSize < 1024) {
                sizeString = QString("%1B").arg(fileSize);
            } else if (fileSize < 1024 * 1024) {
                sizeString = QString("%1KB").arg(QString::number(fileSize / 1024.0f, 10, 1));
            } else {
                sizeString = QString("%1MB").arg(QString::number(fileSize / 1024 / 1024.0f, 10, 1));
            }
            QString trackInfo = "%1,%2,%3";
            trackInfo = trackInfo.arg(track->trackDefinition).arg(track->vodFormat).arg(sizeString);
            ui->comboTrack->insertItem(index, trackInfo);
            QVariant q(track->trackIndex);
            ui->comboTrack->setItemData(index, q);
            if (index == 0) {
                mMediaDownloader->selectTrack(track->trackIndex);
            }
        }
    }
    ui->comboTrack->blockSignals(false);
}

void AlivcDownloadPrepareDlg::OnErrorUI(QString error)
{
    QMessageBox::information(NULL, "Error", error, QMessageBox::Ok, QMessageBox::NoButton);
}

void AlivcDownloadPrepareDlg::InitEvent()
{
    connect(this, SIGNAL(SgnPrepared()), this, SLOT(OnPreparedUI()));
    connect(this, &AlivcDownloadPrepareDlg::SgnError, this, &AlivcDownloadPrepareDlg::OnErrorUI);
}

void AlivcDownloadPrepareDlg::EnableStsControls(bool enable)
{
    ui->btnGetStsInfo->setEnabled(enable);
    ui->editStsUrl->setEnabled(enable);
    ui->editAccessKeyId->setEnabled(enable);
    ui->editAccessKeySecret->setEnabled(enable);
    ui->editSecurityToken->setEnabled(enable);
    ui->editExpiration->setEnabled(enable);
}

void AlivcDownloadPrepareDlg::EnablePlayAuthControls(bool enable)
{
    ui->btnGetPlayAuthInfo->setEnabled(enable);
    ui->editPlayAuthUrl->setEnabled(enable);
    ui->editPlayAuth->setEnabled(enable);
}