#include "AlivcPlayListDlg.h"
#include <QHBoxLayout>
#include <QHeaderView>
#include <QJsonArray>
#include <QJsonDocument>
#include <QJsonObject>
#include <QLabel>
#include <QMessageBox>

AlivcPlayListDlg::AlivcPlayListDlg(QWidget *parent) : QDialog(parent)
{
    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
    InitLayout();
    InitEvent();
    InitData();
}

AlivcPlayListDlg::~AlivcPlayListDlg()
{}

void AlivcPlayListDlg::OnPlayButtonClicked(bool checked)
{
    if (mPlayListModel == nullptr) {
        return;
    }

    QModelIndex index = mPlayListView->currentIndex();
    if (!index.isValid()) {
        QMessageBox::warning(this, QStringLiteral(""), QStringLiteral("ѡҪŻصƵ"));
        return;
    }

    AlivcVideoInfo videoInfo;
    mPlayListModel->GetVideoInfoByIndex(index.row(), videoInfo);
    if (videoInfo.videoId.isEmpty()) {
        QMessageBox::warning(this, QStringLiteral(""), QStringLiteral("ƵvidϢΪ"));
        return;
    }

    mCurrentVid = videoInfo.videoId;
    accept();
}

void AlivcPlayListDlg::InitLayout()
{
    QLabel *titleLabel = new QLabel(QStringLiteral("Ƶб"));
    QHBoxLayout *titleLayout = new QHBoxLayout();
    titleLayout->addWidget(titleLabel);
    titleLayout->addStretch();

    mPlayListView = new QTableView();
    mPlayListView->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    mPlayListModel = new AlivcPlayListModel();

    mPlayButton = new QPushButton(QStringLiteral("ѡƵ"));
    QHBoxLayout *bottomLayout = new QHBoxLayout();
    bottomLayout->addStretch();
    bottomLayout->addWidget(mPlayButton);
    bottomLayout->addStretch();

    QVBoxLayout *mainLayout = new QVBoxLayout();
    mainLayout->addLayout(titleLayout);
    mainLayout->addWidget(mPlayListView);
    mainLayout->addLayout(bottomLayout);
    setLayout(mainLayout);
}

void AlivcPlayListDlg::InitEvent()
{
    bool ret = true;
    ret = connect(mPlayButton, &QPushButton::clicked, this, &AlivcPlayListDlg::OnPlayButtonClicked);
}

void AlivcPlayListDlg::InitData()
{
    mGetVideoListApid = new AlivcAppServerBaseAPI(this);
    mGetVideoListApid->GetVideoList([&](int statusCode, QByteArray data) {
        QJsonParseError jsonError;
        QJsonDocument jsonDocument = QJsonDocument::fromJson(data, &jsonError);
        if (jsonError.error != QJsonParseError::NoError) {
            return;
        }
        QVariantMap resultMap = jsonDocument.toVariant().toMap();
        QVariantMap dataMap = resultMap["data"].toMap();
        AlivcPlayListModel *curPlayListModel = new AlivcPlayListModel();
        QList<QVariant> videoInfos = dataMap["videoList"].toList();
        for (auto info : videoInfos) {
            QVariantMap infoMap = info.toMap();
            if (infoMap.isEmpty()) {
                continue;
            }

            AlivcVideoInfo videoInfo;
            videoInfo.status = infoMap["status"].toString();
            videoInfo.title = infoMap["title"].toString();
            videoInfo.creationTime = infoMap["creationTime"].toString();
            videoInfo.coverURL = infoMap["coverUrl"].toString();
            videoInfo.videoId = infoMap["videoId"].toString();
            videoInfo.modificationTime = infoMap["modificationTime"].toString();
            videoInfo.modifyTime = infoMap["modifyTime"].toString();
            videoInfo.duration = infoMap["duration"].toULongLong();
            videoInfo.createTime = infoMap["createTime"].toString();
            videoInfo.size = infoMap["size"].toULongLong();
            curPlayListModel->AddVideoInfo(videoInfo);
        }

        mPlayListView->setModel(curPlayListModel);
        if (mPlayListModel) {
            delete mPlayListModel;
            mPlayListModel = nullptr;
        }
        mPlayListModel = curPlayListModel;
    });
}
