#include "AlivcPlayListModel.h"

const int VIDEO_INFO_TABLE_ROWS = 5;
const QString VIDEO_INFO_TABLE_HEADER_NAME[VIDEO_INFO_TABLE_ROWS] = { QStringLiteral("Vid"), QStringLiteral("Title"), QStringLiteral("Size"),QStringLiteral("Duration"), QStringLiteral("CreateTime") };

AlivcPlayListModel::AlivcPlayListModel(QObject *parent)
    : QAbstractTableModel(parent)
{
}

AlivcPlayListModel::~AlivcPlayListModel()
{
}

int AlivcPlayListModel::rowCount(const QModelIndex &parent) const
{
  return mVideoInfos.size();
}

int AlivcPlayListModel::columnCount(const QModelIndex &parent) const
{
  return VIDEO_INFO_TABLE_ROWS;
}

QVariant AlivcPlayListModel::data(const QModelIndex &index, int role) const
{
  if (!index.isValid())
  {
    return QVariant();
  }

  if (index.row() >= mVideoInfos.size())
  {
    return QVariant();
  }

  if (Qt::TextAlignmentRole == role)
  {
    return int(Qt::AlignHCenter | Qt::AlignVCenter);
  }
  else if (Qt::DisplayRole == role)
  {
    if (index.column() == VideoInfoItemVideoIdRow)
    {
      return mVideoInfos[index.row()].videoId;
    }
    else if (index.column() == VideoInfoItemTitleRow)
    {
      return mVideoInfos[index.row()].title;
    }
    else if (index.column() == VideoInfoItemSizeRow)
    {
      return mVideoInfos[index.row()].size;
    }
    else if (index.column() == VideoInfoItemDurationRow)
    {
      return mVideoInfos[index.row()].duration;
    }
    else if (index.column() == VideoInfoItemCreateTimeRow)
    {
      return mVideoInfos[index.row()].createTime;
    }
  }

  return QVariant();
}

QVariant AlivcPlayListModel::headerData(int section, Qt::Orientation orientation, int role) const
{
  if (Qt::DisplayRole != role)
  {
    return QVariant();
  }

  if (orientation == Qt::Horizontal)
  {
    return VIDEO_INFO_TABLE_HEADER_NAME[section];
  }

  return QVariant();
}

void AlivcPlayListModel::AddVideoInfo(AlivcVideoInfo& info)
{
  mVideoInfos.push_back(info);
}

void AlivcPlayListModel::GetVideoInfoByIndex(int index, AlivcVideoInfo& videoInfo)
{
  if (index < 0 || index >= mVideoInfos.size())
  {
    return;
  }

  videoInfo = mVideoInfos[index];
}
