#include "AlivcPlayerPlayAuthDlg.h"
#include "QJsonDocument"
#include <windows.h>

const static QString GetPlayAuthInfoUrl = "https://alivc-demo.aliyuncs.com/player/getVideoPlayAuth";

AlivcPlayerPlayAuthDlg::AlivcPlayerPlayAuthDlg(QWidget *parent, const QString &vid)
    : QDialog(parent), mVid(vid), ui(new Ui::AlivcPlayerPlayAuthDlg)
{
    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
    ui->setupUi(this);
    QString url = GetPlayAuthInfoUrl;
    if (!mVid.isEmpty()) {
        url.append("?videoId=");
        url.append(mVid);
    }
    ui->editPlayAuthUrl->setText(url);
}

AlivcPlayerPlayAuthDlg::~AlivcPlayerPlayAuthDlg()
{
    if (mGetPlayAuthInfoApi) {
        delete mGetPlayAuthInfoApi;
        mGetPlayAuthInfoApi = nullptr;
    }
}

AlivcPlayAuthInfo AlivcPlayerPlayAuthDlg::GetPlayAuthInfo()
{
    AlivcPlayAuthInfo ret;
    ret.vid = ui->editVid->text();
    ret.playAuth = ui->editPlayAuth->text();
    ret.region = ui->editRegion->text();
    return ret;
}

void AlivcPlayerPlayAuthDlg::on_btnGetPlayAuthInfo_clicked()
{
    if (mGetPlayAuthInfoApi == nullptr) {
        mGetPlayAuthInfoApi = new AlivcAppServerBaseAPI(this);
    }
    mGetPlayAuthInfoApi->Get(ui->editPlayAuthUrl->text(), [&](int statusCode, QByteArray data) {
        QJsonParseError jsonError;
        QJsonDocument jsonDocument = QJsonDocument::fromJson(data, &jsonError);
        if (jsonError.error != QJsonParseError::NoError || !jsonDocument.isObject()) {
            return;
        }

        QVariantMap result = jsonDocument.toVariant().toMap();
        QVariant dataValue = result["data"];
        QVariantMap dataMap = dataValue.toMap();
        ui->editPlayAuth->setText(dataMap["playAuth"].toString());
        QVariantMap metaMap = dataMap["videoMeta"].toMap();
        ui->editVid->setText(metaMap["videoId"].toString());
    });
}

void AlivcPlayerPlayAuthDlg::on_btnDone_clicked()
{
    accept();
}
