//
//  AVDConfig.h
//
//  Created by kongjian.kongian on 2020/06/04.
//

#ifndef AVDConfig_h
#define AVDConfig_h

#include <corecrt_malloc.h>
#include <string.h>

namespace alivc_player {

    typedef struct _AVDConfig {
        /**
        @brief  Maximum timeout time. Default: 15000 milliseconds.
        */
        int timeoutMs;

        /**
        @brief Maximum connection timeout time. Default: 5000 milliseconds.
        */
        int connnectTimoutMs;

        /**
        @brief  Request referer
        */
        char *referer;

        /**
        @brief  User Agent
        */
        char *userAgent;

        /**
        @brief HTTP proxy
        */
        char *httpProxy;

        _AVDConfig()
        {
            referer = nullptr;
            userAgent = nullptr;
            httpProxy = nullptr;
            timeoutMs = 15000;
            connnectTimoutMs = 5000;
        }

        _AVDConfig(const _AVDConfig &config)
        {
            referer = nullptr;
            userAgent = nullptr;
            httpProxy = nullptr;

            if (config.referer) {
                referer = strdup(config.referer);
            }
            if (config.userAgent) {
                userAgent = strdup(config.userAgent);
            }
            if (config.httpProxy) {
                httpProxy = strdup(config.httpProxy);
            }

            timeoutMs = config.timeoutMs;
            connnectTimoutMs = config.connnectTimoutMs;
        }

        _AVDConfig operator=(const _AVDConfig &config)
        {
            if (config.referer) {
                if (referer) {
                    free(referer);
                }
                referer = strdup(config.referer);
            }
            if (config.userAgent) {
                if (userAgent) {
                    free(userAgent);
                }
                userAgent = strdup(config.userAgent);
            }
            if (config.httpProxy) {
                if (httpProxy) {
                    free(httpProxy);
                }
                httpProxy = strdup(config.httpProxy);
            }

            timeoutMs = config.timeoutMs;
            connnectTimoutMs = config.connnectTimoutMs;

            return *this;
        }

        ~_AVDConfig()
        {
            if (referer) {
                free(referer);
            }
            if (userAgent) {
                free(userAgent);
            }
            if (httpProxy) {
                free(httpProxy);
            }
        }

    } AVDConfig;
};// namespace alivc_player

#endif /* AVDConfig_h */
