//
//  AVPMediaInfo.h
//
//  Created by shiping.csp on 2018/11/16.
//  Copyright © 2018年 com.alibaba.AliyunPlayer. All rights reserved.
//

#ifndef AVPMediaInfo_h
#define AVPMediaInfo_h

#include "AVPDef.h"

#define MAX_TRACKCOUNT 36

namespace alivc_player {

    const static int SELECT_AVPTRACK_TYPE_VIDEO_AUTO = -1;
    /**
     @brief track type
     * AVPTRACK_TYPE_VIDEO: video track
     * AVPTRACK_TYPE_AUDIO: audio track
     * AVPTRACK_TYPE_SUBTITLE: subtitle track
     * AVPTRACK_TYPE_MIXED: have auido and video info in this track
     */
    typedef enum _AVPTrackType {
        AVPTRACK_TYPE_VIDEO,
        AVPTRACK_TYPE_AUDIO,
        AVPTRACK_TYPE_SUBTITLE,
        AVPTRACK_TYPE_SAAS_VOD,//track from vod
    } AVPTrackType;


    typedef struct ALIVC_EXTERN _AVPTrackInfo {

        bool valid;

        /**
         @brief track type @see AVPTrackType
         */
        AVPTrackType trackType;

        /**
         @brief track index
         */
        int trackIndex;

        /**
         @brief track definition
         */
        char* trackDefinition;

        /**
         @brief video width
         */
        int videoWidth;

        /**
         @brief video height
         */
        int videoHeight;

        /**
         @brief bitrate
         */
        int trackBitrate;

        /**
         @brief the number of audio channel
         */
        int audioChannels;

        /**
         @brief audio samplerate
         */
        int audioSamplerate;

        /**
         @brief audio sampleFormat
         */
        int audioSampleFormat;

        /**
         @brief audio Language
         */
        char* audioLanguage;

        /**
         @brief subtitle Language
         */
        char* subtitleLanguage;

        /**
         @brief vod file size, int Byte, only meaningful if the trackType is AVPTRACK_TYPE_SAAS_VOD
         */
        int vodFileSize = 0;

        /**
         @brief vod format, only meaningful if the trackType is AVPTRACK_TYPE_SAAS_VOD
         */
        char* vodFormat;

        void clear();
        _AVPTrackInfo();
        _AVPTrackInfo(const _AVPTrackInfo&);
        _AVPTrackInfo& operator=(const _AVPTrackInfo&);
        ~_AVPTrackInfo();

    } AVPTrackInfo;


    typedef struct ALIVC_EXTERN _AVPMediaInfo {
        /**
         @brief media title
         */
        char* title;

        /**
         @brief media cover url
         */
        char* coverURL;

        /**
         @brief media type
         * video audio video|audio
         */
        char* mediaType;

        /**
         @brief media status
         */
        char* status;

        /**
         @brief media duration
         * 单位是秒
         */
        int duration;

        /**
         @brief media transcodeMode
         *转码类型，取值：FastTranscode（上传完成即转码，且转码完成才能播放），NoTranscode（上传完成不转码，且立即能播放），默认取值FastTranscode
         */
        char* transcodeMode;

        /**
         @brief tracks in media info
         */
        AVPTrackInfo* tracks[MAX_TRACKCOUNT];

                /**
                @brief track count
                */
                int trackCount;

        void clear();

        _AVPMediaInfo();
        _AVPMediaInfo(const _AVPMediaInfo&);
        _AVPMediaInfo& operator=(const _AVPMediaInfo&);
        ~_AVPMediaInfo();
    }AVPMediaInfo;
};

#endif /* AVPMediaInfo_h */
