﻿//
//  AVPSource.h
//
//  Created by shiping.csp on 2018/11/16.
//  Copyright (c) 2018 com.alibaba.AliyunPlayer. All rights reserved.
//

#ifndef AVPSOURCE_H
#define AVPSOURCE_H

#include <stdlib.h>
#include <stdint.h>

#include "AVPDef.h"

namespace  alivc_player {

    class ALIVC_EXTERN AVPSource {

    public:
        AVPSource();

        virtual ~AVPSource();

    public:
        void setCoverURL(const char* url);

        void setTitle(const char* title);

        void setQuality(const char* quality);

        char* getCoverURL() const;

        char* getTitle() const;

        char* getQuality() const;

    protected:
        /**
         @brief 封面地址
         */
        char* mCoverURL;

        /**
         @brief 视频标题
         */
        char* mTitle;

        /**
         @brief 视频清晰度
         */
        char* mQuality;

    };

    class ALIVC_EXTERN AVPUrlSource : public AVPSource {

    public:
        AVPUrlSource();

        ~AVPUrlSource();

    public:
        void setUrl(const char* url);

        char* getUrl() const;

    private:
        char* mUrl;
    };


    class ALIVC_EXTERN AVPVidStsSource : public AVPSource {
    public:
        AVPVidStsSource();

        ~AVPVidStsSource();

    public:
        /**
         @brief 用vid和sts来播放视频，临时AccessKeyId、AccessKeySecret和SecurityToken：开启RAM授权，并通过STS授权系统提供的OpenAPI或SDK获取的AccessKeyId、AccessKeySecret和SecurityToken，用于播放和下载请求参数明细：https://help.aliyun.com/document_detail/28788.html?spm=5176.doc28787.6.706.2G5SLS
         @param vid 视频播放的vid标识
         @param accessKeyId 用户sts的accessKey ID
         @param accessKeySecret 用户sts的accessKey secret
         @param securityToken 用户sts的token信息
         @param region 用户sts的region信息
         */
        void initWithVid(const char *vid, const char *accessKeyId, const char *accessKeySecret, const char *securityToken,
                         const char *region, const char *playConfig);

        char* getVid() const;
        char* getAccessKeyId() const;
        char* getAccessKeySecret() const;
        char* getSecurityToken() const;
        char* getRegion() const;
        char* getPlayConfig() const;

    private:
        /**
         @brief vid
         */
        char* mVid;

        /**
         @brief accessKeyId
         */
        char* mAccessKeyId;

        /**
         @brief accessKeySecret
         */
        char* mAccessKeySecret;

        /**
         @brief securityToken
         */
        char* mSecurityToken;

        /**
         @brief region
         */
        char* mRegion;

        /**
         @brief play config
         */
        char *mPlayConfig;
    };

    class ALIVC_EXTERN AVPVidAuthSource : public AVPSource {
    public:
        AVPVidAuthSource();

        ~AVPVidAuthSource();

    public:
        void initWithVid(const char *vid, const char *playAuth, const char *region, const char *playConfig);

        char* getVid() const;
        char* getPlayAuth() const;
        char* getRegion() const;
        char* getPlayConfig() const;

    private:
        /**
         @brief vid
         */
        char* mVid;

        /**
         @brief playAuth
         */
        char* mPlayAuth;

        /**
         @brief region
         */
        char* mRegion;

        /**
         @brief play config
         */
        char *mPlayConfig;
    };

    class ALIVC_EXTERN AVPVidMpsSource : public AVPSource {

    public:
        AVPVidMpsSource();

        ~AVPVidMpsSource();

    public:
        /**
         @brief 用vid和MPS信息来播放视频，视频转码服务用户使用播放方式。部分参数参考:https://help.aliyun.com/document_detail/53522.html?spm=5176.doc53534.2.5.mhSfOh
         @param vid 视频播放的vid标识
         @param accId 用户sts的accessKey ID
         @param accSecret 用户sts的accessKey secret
         @param stsToken 用户sts的token信息
         @param authInfo 用户authInfo信息
         @param region 用户region信息
         @param playDomain 用户playDomain信息
         @param mtsHlsUriToken 用户mtsHlsUriToken信息
         */
        void initWithVid(const char* vid,
                         const char* accId,
                         const char* accSecret,
                         const char* stsToken,
                         const char* authInfo,
                         const char* region,
                         const char* playDomain,
                         const char* mtsHlsUriToken);

        char* getVid() const;
        char* getAccId() const;
        char* getAccSecret() const;
        char* getStsToken() const;
        char* getAuthInfo() const;
        char* getRegion() const;
        char* getPlayDomain() const;
        char* getMtsHlsUriToken() const;

    private:
        /**
         @brief vid
         */
        char* mVid;

        /**
         @brief accId
         */
        char* mAccId;

        /**
         @brief accSecret
         */
        char* mAccSecret;

        /**
         @brief stsToken
         */
        char* mStsToken;

        /**
         @brief authInfo
         */
        char* mAuthInfo;

        /**
         @brief region
         */
        char* mRegion;

        /**
         @brief playDomain
         */
        char* mPlayDomain;

        /**
         @brief mtsHlsUriToken
         */
        char* mMtsHlsUriToken;
    };
};

#endif /* AVPSOURCE_H */
